function f = st_features_itq(image,windows,func,fargs,itq)
% Applies ITQ to any feature function output.
% See also http://www.unc.edu/~yunchao/itq.htm .
%
% arguments:
%   image = rgb uint8 image, or cell of ~
%   windows = array of windows, or cell of ~
%   func = function handle to st_features_*
%   fargs = cell of additional argumetns to func (appended at the end)
%   itq = struct with ITQ configuration
%     mean = trained data mean
%     pc = trained principal components
%     R = trained rotation
%
% returns:
%   f = features after ITQ mapping
%     saved as rows of uint64's

if iscell(image)
	f = map('c','par',@compute,'cc---',{image,windows,func,fargs,itq});
else
	f = compute(image,windows,func,fargs,itq);
end;


function f = compute(image,windows,func,fargs,itq)

f = func(image,windows,fargs{:});
f = ( bsxfun(@minus,f,itq.mean) * itq.pc * itq.R) >= 0;
f = logicals_to_bits(f,64,2);

