function s = st_score_iou(gt,m)
% Computes the intersection over union (iou) score.
%
% arguments:
%   gt = groundtruth, logical image, or cell of ~
%   m = prediction, logical image, or cell of ~
%
% returns:
%   s = IoU score in range [0..1]

if iscell(gt)
	assert(iscell(m));
	s = map('a','for',@compute,'cc',{gt,m});
else
	s = compute(gt,m);
end;


function s = compute(gt,m)
% For one image only.

gt = gt(:);
m = m(:);

i = sum(gt & m); % intersection
u = sum(gt | m); % union

if u==0
	% no pixel in both gt and m, perfect prediction
	s = 1;
else
	s = i/u;
end;

