function [p r f] = st_score_pr(gt,m)
% Computes scores based on precision/recall.
% For multiple images, the score is computed on the joined set of pixels.
%
% arguments:
%   gt = groundtruth, logical image, or cell of ~
%   m = prediction, logical image, or cell of ~
%
% returns
%   p = precision
%   r = recall
%   f = f-measure = 2*(p*r)/(p+r), p and r evenly weighted

if iscell(gt)
	assert(iscell(m));
	gt = reshape(gt,[],1);
	gt = map('a','for',@(d) d(:),'c',{gt});
	m = reshape(m,[],1);
	m = map('a','for',@(d) d(:),'c',{m});
else
	gt = gt(:);
	m = m(:);
end;

tp = sum(m & gt);
fp = sum(m & ~gt);
fn = sum(~m & gt);

if tp+fp>0
	p = tp/(tp+fp);
else
	p = 1; % no positives predictions at all, so all positives are correct
end;

if tp+fn>0
	r = tp/(tp+fn);
else
	r = 1; % there are no positives in the ground truth, so all are recalled
end;

if p+r>0
	f = 2*(p*r)/(p+r);
else % there are no true positives
	f = 0; % but there were positives
end;

