function source = st_source(masksize,masks,windows,features)
% Builds a source for segmentation transfer.
%
% arguments
%   masksize = square size of masks prepared for transfer
%   masks = cell of masks of full source images (logicals)
%   windows = cell of windows
%   features = cell of features
%
% returns
%   source = struct of source data

n = length(masks);
assert(n==length(windows));
assert(n==length(features));

ns = map_c2a(@(w) size(w,1),windows);
assert(all(ns==map_c2a(@(f) size(f,1),features)));

N  = sum(ns);
D = size(features{1},2);
M = masksize;

source.N = N;
source.D = D;
source.M = M;

source.masks = false(M,M,N);
source.windows = zeros(N,4);
source.features = zeros(N,D,class(features{1}));
source.images = zeros(N,1);

j = 1;
for i = 1:n
	for w = 1:ns(i)
		source.masks(:,:,j) = resize_mask(masks{i},windows{i}(w,:),M);
		source.features(j,:) = features{i}(w,:);
		source.images(j) = i;
		j = j+1;
	end;
end;
assert(j==N+1);


function r = resize_mask(m,w,s)

H = size(m,1);
W = size(m,2);

w(1) = w(1)*(W-1)+1;
w(2) = w(2)*(H-1)+1;
w(3) = w(3)*(W-1)+1;
w(4) = w(4)*(H-1)+1;

y = round(linspace(w(2),w(4),s));
x = round(linspace(w(1),w(3),s));

r = m(y,x);

