function windows = st_windows(image,count)
% Finds candidate windows in images.
% Using Objectness (see Alexe et al. What is an object? CVPR 2010).
%
% arguments
%   image = single rgb uint8 image or cell of images
%   count = number of candidate windows to produce per image
%
% returns
%   windows = count rows of [xmin ymin xmax ymax]
%     the coordinates are normalized to [0..1]
%     for multiple input images, output is a cell of window arrays

if iscell(image)
	windows = map('c','par',@compute,'c-',{image,count});
else
	windows = compute(image,count);
end;


function windows = compute(image,count)
% For just one image.

windows = runObjectness(image,count); % returns [xmin ymin xmax ymax score]

H = size(image,1);
W = size(image,2);

windows(:,1) = (windows(:,1)-1)/(W-1);
windows(:,2) = (windows(:,2)-1)/(H-1);
windows(:,3) = (windows(:,3)-1)/(W-1);
windows(:,4) = (windows(:,4)-1)/(H-1);

