
%% Collect the training data.
images = ...; % cell of RGB uint8 images
segmentations = ...; % cell of logical images (ground truth)

%% Get the windows for transfer and compute their feature representations.
windows = st_windows(images);
features = st_featrues_gist(images,windows);

%% Construct a "source" for segmentation transfer.
source = st_source(50,segmentations,windows,features);

%% This source can be used for segmentation transfer.
%  Only the source is needed, the other data can be cleared.
clear images segmentations windows features;

%% To segment one test image, transfer masks from source
%  and then segment it using the transferred masks.
image = ...; % RGB uint8 image
windows = st_windows(image,100);
features = st_features_gist(image,windows);
weights = ones(1,100)/100;
mask = st_transfer(source,50,windows,features,@distance_L2,weights);
prediction = st_segment(image,mask,0.2,20);

%% Most functions work also with a set of images.
%  Thus, to segment a set of test images:
images = ...; % cell of RGB uint8 images
windows = st_windows(images,100);
features = st_features_gist(images,windows);
weights = ones(1,100)/100;
masks = st_transfer(source,50,windows,features,@distance_L2,weights);
predictions = st_segment(images,masks,0.2,20);

%% If you have ground truth for the test set, compute the scores of the prediction.
segmentations = ...; % cell of logical images (ground truth for test set)
ious = st_score_iou(segmentations,predictions);
accs = st_score_acc(segmentations,predictions);
[~,~,f1] = st_score_pr(segmentations,predictions);

%% For large-scale data, the feature representation of all windows is
%  too large to keep in memory.
%  Instead, you should use:
itq = load_var('data/sets/voc10/train_itq.mat');
features = st_features_itq(images,windows,@st_feature_gist,{},itq);
% This ITQ is trained on the voc10 training set.
% See st_itq.m to train your own ITQ configuration.

%% Further, to speed up the segmentation transfer, we trained a
%  weighted model for transfer. Instead of using 100 nearest neighbors,
%  transfer is now a weighted sum with less neighbors.
weights = load_var('data/sets/voc10/train_weights.mat');
masks = st_transfer(source,50,windows,features,@distance_hamming,weights);
% (Note also the use of @distance_hamming for ITQ instead of @distance_L2 for Gist.)
% (Note in [2] we use HOG features, but we didn't add a HOG implementation here.)
